/* 
 * Copyright (C) 2001 Igor Sysoev
 *
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 /

/*
MODULE-DEFINITION-START
Name: realip_module
ConfigStart
ConfigEnd
MODULE-DEFINITION-END
*/

#include "httpd.h"
#include "http_config.h"


#define MOD_REALIP_VER   "mod_realip/1.0"


typedef enum {
    REALIP_POSTREAD = 1,
    REALIP_HEADERS,
    REALIP_FIXUP,
    REALIP_OFF
} realip_phase;

typedef enum {
    REALIP_X_REAL_IP = 0,
    REALIP_X_FWD_FOR 
} realip_header;

typedef struct {
    realip_phase   phase;
    realip_header  header;
} server_conf;

module realip_module;


static void set_real_remote_ip(request_rec *r, realip_header header)
{
    char        *x_real_ip;
    const char  *x_fwd_for;

    if (header == REALIP_X_REAL_IP) {
        if (x_real_ip = (char *)ap_table_get(r->headers_in, "X-Real-IP"))
            r->connection->remote_ip = x_real_ip;
        return;
    }

    if (x_fwd_for = ap_table_get(r->headers_in, "X-Forwarded-For")) {
        char *token, *last_ip;

        while (*x_fwd_for && (token = ap_get_token(r->pool, &x_fwd_for, 0))) {
            /* ap_get_token() bug */
            if (*x_fwd_for == ',' || *x_fwd_for == ';')
                x_fwd_for++;

            last_ip = token;
        }

        r->connection->remote_ip = last_ip;
    }
}


static int realip_postread(request_rec *r)
{
    server_conf *conf = (server_conf *)
             ap_get_module_config(r->server->module_config, &realip_module);

    if (conf->phase == REALIP_POSTREAD) {
        set_real_remote_ip(r, conf->header);
    }

    return DECLINED;
}

static int realip_header_parser(request_rec *r)
{
    server_conf *conf = (server_conf *)
             ap_get_module_config(r->server->module_config, &realip_module);

    if (conf->phase == REALIP_HEADERS) {
        set_real_remote_ip(r, conf->header);
    }

    return DECLINED;
}

static int realip_fixups(request_rec *r)
{
    server_conf *conf = (server_conf *)
             ap_get_module_config(r->server->module_config, &realip_module);

    if (conf->phase == REALIP_FIXUP) {
        set_real_remote_ip(r, conf->header);
    }

    return DECLINED;
}


static void *realip_config(pool *p, server_rec *s)
{
    return (void *)ap_pcalloc(p, sizeof(server_conf));
}

static void *realip_merge(pool *p, void *base, void *new)
{
    server_conf *merged = ap_pcalloc(p, sizeof(server_conf));
    server_conf *parent = (server_conf *) base;
    server_conf *child = (server_conf *) new;

    merged->phase = child->phase ? child->phase : parent->phase;
    merged->header = child->header ? child->header : REALIP_X_REAL_IP;

    return (void *)merged;
}



static const char *set_realip(cmd_parms *cmd, void *dummy,
                              char *phase, char*header)
{
    server_conf *conf = (server_conf *)
             ap_get_module_config(cmd->server->module_config, &realip_module);


    if (strcmp(phase, "postread") == 0) {
        conf->phase = REALIP_POSTREAD;

    } else if (strcmp(phase, "header") == 0) {
        conf->phase = REALIP_FIXUP;

    } else if (strcmp(phase, "fixups") == 0) {
        conf->phase = REALIP_FIXUP;

    } else if (strcmp(phase, "off") == 0) {
        conf->phase = REALIP_OFF;

    } else {
        return "RealIP phase can be 'postread', 'header' or 'fixups' only";
    }

    if (header) {
        if (strcmp(header, "xfwd") == 0)
            conf->header = REALIP_X_FWD_FOR;
        else
            return "RealIP header can be 'xfwd' only";
    }

    return NULL;
}


static const command_rec realip_cmds[] =
{
    {"RealIP", set_realip, NULL, RSRC_CONF, TAKE12,
     "set phase to set real remote ip"},

    {NULL}
};


module realip_module =
{
    STANDARD_MODULE_STUFF,
    NULL,                             /* module initializer */
    NULL,                             /* per-directory config creator */
    NULL,                             /* dir config merger */
    realip_config,                    /* server config creator */
    realip_merge,                     /* server config merger */
    realip_cmds,                      /* command table */
    NULL,                             /* [9] list of handlers */
    NULL,                             /* [2] filename-to-URI translation */
    NULL,                             /* [5] check/validate user_id */
    NULL,                             /* [6] check user_id is valid *here* */
    NULL,                             /* [4] check access by host address */
    NULL,                             /* [7] MIME type checker/setter */
    realip_fixups,                    /* [8] fixups */
    NULL,                             /* [10] logger */
    realip_header_parser,             /* [3] header parser */
    NULL,                             /* process initializer */
    NULL,                             /* process exit/cleanup */   
    realip_postread,                  /* [1] post read_request handling */
}; 
